// Statistics.hh

// Created by John Zweck, Feb 7th, 2003.
// Adapted from the StatisticsOfArray class in OCS

/*

 Every class must have a class definition which has the declarations
 of all the functions and methods that make up the class. 

 The definition of MyClass is given in the header (or include) file, 
 MyClass.hh.

 Methods and data can be declared to be either public or private.

 Any method in the class has access to any of the private or public data
  members of the class. 

 An applications program has access to the public methods and data 
  of an instance of the class.

 Generally speaking the definition of the methods in the class goes in
 the class source file, MyClass.cc.

 However, if a method is very short it is sometimes defined within the class
 definition.

 The idea behind putting the definition of a class in the header file
 and the definitions of the methods in the class in the source file
 is that if the code is well documented, an applications programmer
 should only need to know the contents of the header file and not
 have to worry about the way the methods in the class are implemented
 in the source code file.

 Class definitions generally have the syntax

 class MyClass
  {
    public:

     // Declaration of the constructors and destructors

     MyClass(....);
     ~MyClass();

     // Declaration of public methods 

     int MyPublicMethod(.......);

     // Declaration of public data (if any) 
   
     double MyPublicData;

    private:

     // Declaration of private methods (if any) 

     int MyPrivateMethod(.......);

     // Declaration of private data goes here

    double MyPrivateData;

  }; // Note the final ; !!!!

*/


#ifndef _STATISTICS_HH
#define _STATISTICS_HH

//! 
//!    The Statistics class computes simple statistical quantities
//!    from a double array of samples provided by the user.  
//!
//!    This rest of this file contains the definition of the class Statistics.
//!
//!    It contains the declarations of the member functions and data members
//!    of the class. 
//!
//!    However it does not include initialization of the data.
//!    That must be done in the class constructor


class Statistics
{

  // ### Public Methods ###

 public:

  //! The class constructor


  /*! 

      General facts about constructors:

      -# The constructor initializes (all) the data members of the class. 
      -# It must have the same name as the class.
      -# A class can have more than one constructor, but the input parameters
         of two different constructors must be different
      -# A constructor does not have a return type
      -# The constructor must always be a public method! 
      -# The constructor can (and often does) call other member functions
      -# In an application program you create a new instance of a class
         by calling one of the class's constructors. 
      -# Each separate instance of the class is called an object. 


      The input parameters to the Statistics class are

      \param arrayLengthIn: The length of the array which stores the 
                            samples
      \param arrayIn: The array in which the samples are stored

      The constructor uses the input parameters to initialize

      the private data members Statistics::arrayLength and Statistics::array

      Then it calls other member functions 
      to compute the values of the private data members

      -# Statistics::minimum
      -# Statistics::maximum
      -# Statistics::mean
      -# Statistics::variance
      -# Statistics::standardDeviation
      -# Statistics::median

      

     Question: What is wrong with the declaration

       Statistics(int arrayLength, double *array);

  */


  Statistics(int arrayLengthIn,  double *arrayIn);


  //! The class destructor.
  /*! 

      General facts about destructors:

      -# The destructor is called when an object is destroyed
      -# The destructor is always named ~ClassName(). 
      -# It has no parameter list or return type
      -# The destructor of an object is called automatically 
         when the program that is executing leaves the scope 
         in which the object was created
      -# The most common use of a destructor is to delete memory
         that was dynamically allocated by the constructor.
      -# In OCS the destructor ~RanNumGen() saves the current state
         of the RNG to a file. 

      The Statistics destructor deletes the array Statistics::sortedArray

  */

  ~Statistics();

  //! Returns the mimimum value of the samples

  double GetMinimum() {return minimum;}; // Both declaration and definition!! 

  double GetMaximum() {return maximum;};
  double GetMean() {return mean;};
  double GetStdDev() {return standardDeviation;};
  double GetVariance() {return variance;};
  double GetMedian() {return median;};

 // ### Private Methods ###

private:

  //! Computes the mimimum value of the samples

  void ComputeMinimum(void);
  void ComputeMaximum(void);
  void ComputeMean(void);
  void ComputeMeanOfSquares(void);
  void ComputeVariance(void);
  void ComputeMedian(void);
 
  //! Used in calculation of median

  void SelectionSort(void);

  //! Used in calculation of median

  int MaximumIndex(int SubLength);

  // ### Private Data ###

 private:

  //! The number of samples. 

  int arrayLength;

  //! Array to store values of samples. Must be initialized by user 
  //! in application code

  double * array;

  //! Used in calculation of median

  double * sortedArray;

  //! Variable which holds the minimum value of the samples

  double minimum;
  double maximum;
  double mean,meanOfSquares,standardDeviation,variance;
  double median;


}; // ## end of definition of class  Statistics


#endif /* _STATISTICS_HH */


